import json
import logging
import ssl

import aiohttp
from aiohttp import WSMessage

from . import names
from .unsee import UnseeImage

_UNSEE_WEBSOCKET_URL = 'wss://ws.unsee.cc/{}/'


class Client:

    def __init__(self, session=None, out_path='.', group_album=True):
        """
        :param session: http session
        :type session: aiohttp.ClientSession
        :param out_path: output path
        :type out_path: str
        :param group_album: should images be grouped in folders per album
        :type group_album: bool
        """
        if session:
            self.session = session
        else:
            self.session = aiohttp.ClientSession()
        self.out_path = out_path
        self.group_album = group_album
        self.token = None

    async def __aenter__(self):
        self._did_enter_with = True
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session and self._did_enter_with:
            await self.session.close()

    async def download_album(self, album_id):
        unsee_name = names.get_random()
        ws_params = '?album={}&name={}'.format(album_id, unsee_name)
        ssl_context = ssl.create_default_context()

        # Settings WS
        async with self.session.ws_connect(_UNSEE_WEBSOCKET_URL.format('settings') + ws_params,
                                           ssl=ssl_context) as ws_settings:
            await ws_settings.send_str('{}')
            data = await ws_settings.receive()

            settings = json.loads(data.data, encoding='utf-8')

            logging.info('[ws_settings] {}'.format(settings))

        images_info = []

        # PubSub WS
        async with self.session.ws_connect(_UNSEE_WEBSOCKET_URL.format('pubsub') + ws_params,
                                           ssl=ssl_context) as ws_pubsub:
            data: WSMessage
            async for data in ws_pubsub:
                if data.type == aiohttp.WSMsgType.BINARY:
                    message = json.loads(data.data, encoding='utf-8')
                    logging.debug('[ws_pubsub] Received message: {}'.format(message))

                    if message['type'] == 'image':
                        images_info.append(message)
                        logging.info('[ws_pubsub] {}'.format(message))
                    else:
                        logging.debug('[ws_pubsub] Closing socket')
                        await ws_pubsub.close()
                elif data.type == aiohttp.WSMsgType.ERROR:
                    break

        logging.info('Found {} images in album {}'.format(len(images_info), album_id))
        if 'title' in settings and len(settings['title']) > 0:
            print('Found album "{}" ({}) with {} images.'.format(settings['title'], album_id, len(images_info)))
        else:
            print('Found album {} with {} images.'.format(album_id, len(images_info)))

        if len(images_info) <= 0:
            return

        # Imgpush WS
        async with self.session.ws_connect(_UNSEE_WEBSOCKET_URL.format('imgpush') + ws_params,
                                           ssl=ssl_context) as ws_imgpush:
            data: WSMessage
            async for data in ws_imgpush:
                logging.debug('[ws_imgpush] received image (len: {})'.format(len(data.data)))

                # noinspection PyBroadException
                try:
                    image = UnseeImage(album_id, out_path=self.out_path, group_album=self.group_album)
                    image_path = image.write_file_from_blob(data.data)
                    logging.debug('Wrote image {}'.format(image_path))

                    image_info = next(filter(lambda info: info['id'] == image.image_id, images_info), None)
                    if image_info is not None:
                        images_info.remove(image_info)
                except:
                    logging.warning('Failed writing image from album {}'.format(album_id))

                if len(images_info) <= 0:
                    await ws_imgpush.close()



import logging

import aiohttp

from unsee_dl.unsee import UnseeImage

_GRAPHQL_URL = 'https://api.unsee.cc/graphql'


class Client:

    def __init__(self, session=None, out_path='.', group_album=True):
        """
        :param session: http session
        :type session: aiohttp.ClientSession
        :param out_path: output path
        :type out_path: str
        :param group_album: should images be grouped in folders per album
        :type group_album: bool
        """
        if session:
            self.session = session
        else:
            self.session = aiohttp.ClientSession()
        self.out_path = out_path
        self.group_album = group_album
        self.token = None

    async def __aenter__(self):
        self._did_enter_with = True
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session and self._did_enter_with:
            await self.session.close()

    async def anonymous_login(self):
        """
        Login with an anonymous token
        """
        gql_body = {
            'operationName': 'loginAnonymous',
            'variables': {},
            'query': """
    mutation loginAnonymous {
      login {
        ...AuthPayloadFragment
        __typename
      }
    }
    
    fragment AuthPayloadFragment on AuthPayload {
      token
      tokenRefresh
      __typename
    }"""
        }
        async with self.session.post('https://api.unsee.cc/graphql', json=gql_body) as response:
            content = await response.json()
            login_data = content['data']['login']
            self.token = login_data['token']

    async def download_album(self, album_id):
        """
        Download an album from unsee beta
        :param album_id: album id
        :type album_id: str
        """
        async for album_image in self._original_size_images(album_id):
            image = UnseeImage(album_id, album_image['id'], self.out_path, self.group_album)
            await self._download_and_save_image(image, album_image['url'])

    async def _original_size_images(self, album_id):
        """
        Get original size image for the album
        :param album_id: unsee album id
        :type album_id: str
        :return: generator with each image in album
        :rtype: Generator
        """
        headers = {
            'authorization': self.token
        }
        gql_body = {
            'operationName': 'getImages',
            'variables': {
                'filter': {
                    'chat': album_id
                },
                'pagination': {
                    'offset': 0
                }
            },
            'query': """
    query getImages($filter: ImageFilter!, $pagination: Pagination) {
    getImages(filter: $filter, pagination: $pagination) {
     id
     url
     __typename
    }
    }"""
        }
        async with self.session.post(_GRAPHQL_URL, json=gql_body, headers=headers) as response:
            content = await response.json()
            album_items = content['data']['getImages']

            if not album_items or len(album_items) <= 0:
                print(f"No images found in album {album_id}")
                return

            print('Found album {} with {} images.'.format(album_id, len(album_items)))

            for thumb in album_items:
                try:
                    # Get original size image
                    headers = {
                        'authorization': self.token
                    }
                    gql_body = {
                        'operationName': 'getImagesBig',
                        'variables': {
                            'filter': {
                                'chat': album_id,
                                'id': thumb['id']
                            }
                        },
                        'query': """
            query getImagesBig($filter: ImageFilter!) {
              getImages(filter: $filter) {
                id
                url(size: big)
                __typename
              }
            }"""
                    }
                    async with self.session.post(_GRAPHQL_URL, json=gql_body, headers=headers) as response:
                        image_big = await response.json()
                        image_big_items = image_big['data']['getImages']

                    for image_big in image_big_items:
                        yield image_big
                except Exception as ex:
                    logging.warning('Failed writing image from album {}'.format(album_id), exc_info=ex)

    async def _download_and_save_image(self, image, image_url):
        """
        Download and save the image
        :param image: unsee image
        :type image: UnseeImage
        :param image_url: image url
        :type image_url: str
        :return: saved image path
        :rtype: str
        """
        async with self.session.get(image_url) as response:
            image_path = await image.write_file_from_stream(response.content)
            logging.debug('Wrote image {}'.format(image_path))

        return image_path




import random

NAMES = [
    "Aaron",
    "Abasi",
    "Abdalla",
    "Abdu",
    "Abdul",
    "Abe",
    "Abedi",
    "Abel",
    "Abraham",
    "Abubakar",
    "Abuu",
    "Adam",
    "Adamu",
    "Adan",
    "Adili",
    "Adnan",
    "Adolfo",
    "Adolph",
    "Agustin",
    "Ahmad",
    "Ahmed",
    "Ainran",
    "Akbar",
    "Akida",
    "Akili",
    "Akram",
    "Al",
    "Alamini",
    "Alan",
    "Albert",
    "Alberto",
    "Alden",
    "Aldo",
    "Alec",
    "Alejandro",
    "Alex",
    "Alexander",
    "Alfonso",
    "Alfonzo",
    "Alfred",
    "Alfredo",
    "Alhaadi",
    "Ali",
    "Allan",
    "Allen",
    "Alonzo",
    "Alphonse",
    "Alphonso",
    "Alton",
    "Alva",
    "Alvaro",
    "Alvin",
    "Amaad",
    "Amado",
    "Amani",
    "Amar",
    "Ambar",
    "Ame",
    "Amini",
    "Amiri",
    "Amos",
    "Anasa",
    "Anderson",
    "Andre",
    "Andres",
    "Andrew",
    "Andy",
    "Angelo",
    "Anibal",
    "Antar",
    "Anthony",
    "Antoine",
    "Anton",
    "Antone",
    "Antonio",
    "Antony",
    "Antwan",
    "Anwar",
    "Archie",
    "Ariel",
    "Arif",
    "Armand",
    "Armando",
    "Arnold",
    "Arnulfo",
    "Aron",
    "Arron",
    "Art",
    "Arthur",
    "Arturo",
    "Asad",
    "Asani",
    "Asante",
    "Ashur",
    "Athumani",
    "Atif",
    "Aubrey",
    "August",
    "Augustine",
    "Augustus",
    "Aurelio",
    "Austin",
    "Avery",
    "Ayubu",
    "Azaan",
    "Aziz",
    "Azizi",
    "Baakir",
    "Baba",
    "Babechi",
    "Babu",
    "Badawi",
    "Badilini",
    "Badrani",
    "Badru",
    "Baha",
    "Baka",
    "Bakari",
    "Baraka",
    "Barney",
    "Barry",
    "Bart",
    "Barton",
    "Bashiri",
    "Basil",
    "Bausi",
    "Bavual",
    "Beau",
    "Ben",
    "Benito",
    "Benjamin",
    "Bennett",
    "Bennie",
    "Benny",
    "Bernard",
    "Bernardo",
    "Bernie",
    "Berry",
    "Bert",
    "Bilaal",
    "Bilali",
    "Bill",
    "Billy",
    "Blaine",
    "Blair",
    "Blake",
    "Bob",
    "Bobby",
    "Boma",
    "Booker",
    "Boraafya",
    "Boris",
    "Boyd",
    "Brad",
    "Bradford",
    "Bradley",
    "Bradly",
    "Brady",
    "Brain",
    "Branden",
    "Brandon",
    "Brant",
    "Brendan",
    "Brent",
    "Brenton",
    "Bret",
    "Brett",
    "Brian",
    "Brice",
    "Brock",
    "Brooks",
    "Bruce",
    "Bruno",
    "Bryan",
    "Bryant",
    "Bryce",
    "Bryon",
    "Bud",
    "Buddy",
    "Buford",
    "Burhani",
    "Burl",
    "Burt",
    "Burton",
    "Bushiri",
    "Byron",
    "Caleb",
    "Calvin",
    "Cameron",
    "Carl",
    "Carlo",
    "Carlos",
    "Carlton",
    "Carmelo",
    "Carmine",
    "Carroll",
    "Carson",
    "Carter",
    "Cary",
    "Cecil",
    "Cedric",
    "Cesar",
    "Chacha",
    "Chad",
    "Chadwick",
    "Chaga",
    "Chance",
    "Chandu",
    "Chang",
    "Charles",
    "Charley",
    "Charlie",
    "Chase",
    "Chauncey",
    "Chega",
    "Cheja",
    "Chester",
    "Chi",
    "Chilemba",
    "Chimalsi",
    "Chitundu",
    "Chiumbo",
    "Christoper",
    "Christopher",
    "Chuck",
    "Clair",
    "Clarence",
    "Clark",
    "Claude",
    "Claudio",
    "Clay",
    "Clayton",
    "Clement",
    "Cleveland",
    "Cliff",
    "Clifford",
    "Clifton",
    "Clint",
    "Clinton",
    "Clyde",
    "Cody",
    "Colby",
    "Cole",
    "Coleman",
    "Colin",
    "Collin",
    "Conrad",
    "Corey",
    "Cornelius",
    "Cornell",
    "Cory",
    "Coy",
    "Craig",
    "Cruz",
    "Curt",
    "Curtis",
    "Cyril",
    "Cyrus",
    "Dada",
    "Dahoma",
    "Dajan",
    "Daktari",
    "Dallas",
    "Dalton",
    "Damian",
    "Damien",
    "Damion",
    "Damon",
    "Damu",
    "Dan",
    "Dane",
    "Danial",
    "Daniel",
    "Dannie",
    "Danny",
    "Dante",
    "Daraja",
    "Daren",
    "Darin",
    "Dario",
    "Darius",
    "Darnell",
    "Darrel",
    "Darrell",
    "Darren",
    "Darrin",
    "Darryl",
    "Darweshi",
    "Darwin",
    "Daryl",
    "Dau",
    "Daud",
    "Daudi",
    "Dave",
    "David",
    "Davis",
    "Dean",
    "Deandre",
    "Delbert",
    "Delmar",
    "Delmer",
    "Demetrius",
    "Denis",
    "Dennis",
    "Denny",
    "Denver",
    "Deon",
    "Derek",
    "Derick",
    "Derrick",
    "Desmond",
    "Devin",
    "Devon",
    "Dewayne",
    "Dewey",
    "Dewitt",
    "Dexter",
    "Dhoruba",
    "Dick",
    "Diego",
    "Dillon",
    "Dini",
    "Dino",
    "Dion",
    "Dirk",
    "Dogo",
    "Domingo",
    "Dominic",
    "Dominick",
    "Don",
    "Donald",
    "Donn",
    "Donnell",
    "Donnie",
    "Donny",
    "Donovan",
    "Dorian",
    "Doug",
    "Douglas",
    "Doyle",
    "Drew",
    "Duane",
    "Dudley",
    "Dumisha",
    "Duncan",
    "Dustin",
    "Dusty",
    "Dwayne",
    "Dwight",
    "Dylan",
    "Earl",
    "Earle",
    "Earnest",
    "Ed",
    "Eddie",
    "Eddy",
    "Edgar",
    "Edgardo",
    "Edmond",
    "Edmund",
    "Eduardo",
    "Edward",
    "Edwardo",
    "Edwin",
    "Efrain",
    "Efren",
    "Ekevu",
    "Elbert",
    "Eldon",
    "Elewa",
    "Eli",
    "Elias",
    "Elijah",
    "Elimu",
    "Eliseo",
    "Elliot",
    "Elliott",
    "Ellis",
    "Elmer",
    "Elmo",
    "Eloy",
    "Elton",
    "Elvin",
    "Elvis",
    "Elwood",
    "Emanuel",
    "Emerson",
    "Emery",
    "Emil",
    "Emile",
    "Emilio",
    "Emmanuel",
    "Emmett",
    "Emory",
    "Enrique",
    "Erevu",
    "Eric",
    "Erich",
    "Erick",
    "Erik",
    "Ernest",
    "Ernesto",
    "Ernie",
    "Errol",
    "Ervin",
    "Erwin",
    "Esteban",
    "Ethan",
    "Eugene",
    "Eugenio",
    "Evan",
    "Everett",
    "Everette",
    "Ezra",
    "Fabian",
    "Fadhili",
    "Fahim",
    "Fakhri",
    "Faki",
    "Fakihi",
    "Faqihi",
    "Faraji",
    "Farhani",
    "Farid",
    "Faruki",
    "Fathi",
    "Faustino",
    "Fauzi",
    "Federico",
    "Fehed",
    "Feisal",
    "Felipe",
    "Felix",
    "Ferdinand",
    "Fernando",
    "Feruzi",
    "Fidel",
    "Fikirini",
    "Fletcher",
    "Floyd",
    "Fogo",
    "Forest",
    "Forrest",
    "Foster",
    "Francisco",
    "Frank",
    "Franklin",
    "Fred",
    "Freddie",
    "Freddy",
    "Frederic",
    "Frederick",
    "Fredric",
    "Fredrick",
    "Freeman",
    "Fritz",
    "Fuad",
    "Fumo",
    "Fumu",
    "Fundikira",
    "Funga",
    "Furaha",
    "Gabriel",
    "Gacoki",
    "Gakere",
    "Gakuru",
    "Galen",
    "Garland",
    "Garrett",
    "Garry",
    "Garth",
    "Gary",
    "Gatete",
    "Gathee",
    "Gathii",
    "Gatimu",
    "Gavin",
    "Genaro",
    "Gene",
    "Geoffrey",
    "George",
    "Gerald",
    "Gerard",
    "Gerardo",
    "German",
    "Gerry",
    "Gethii",
    "Ghalib",
    "Ghaniy",
    "Gharib",
    "Gheilani",
    "Ghofiri",
    "Gichinga",
    "Gicicio",
    "Gikuyu",
    "Gil",
    "Gilbert",
    "Gilberto",
    "Gino",
    "Giovanni",
    "Githinji",
    "Gitonga",
    "Glen",
    "Glenn",
    "Gonzalo",
    "Gordon",
    "Grady",
    "Graham",
    "Grant",
    "Greg",
    "Gregg",
    "Gregorio",
    "Gregory",
    "Grover",
    "Guillermo",
    "Gus",
    "Gustavo",
    "Guy",
    "Haamid",
    "Habib",
    "Habibu",
    "Hafidh",
    "Hafiz",
    "Haidar",
    "Hainidi",
    "Haji",
    "Haki",
    "Hakim",
    "Hal",
    "Hali",
    "Hamadi",
    "Hamdaan",
    "Hami",
    "Hamidi",
    "Hamisi",
    "Hamza",
    "Hank",
    "Hans",
    "Haoniyao",
    "Harambee",
    "Harlan",
    "Harley",
    "Harold",
    "Harris",
    "Harrison",
    "Harry",
    "Harun",
    "Harvey",
    "Hasan",
    "Hasani",
    "Hashil",
    "Hashim",
    "Hasnuu",
    "Hatari",
    "Hauli",
    "Heath",
    "Hector",
    "Hekima",
    "Henry",
    "Herbert",
    "Heri",
    "Heriberto",
    "Herman",
    "Herschel",
    "Hershel",
    "Heshima",
    "Heshimu",
    "Hilali",
    "Hilario",
    "Himidi",
    "Hiram",
    "Hisham",
    "Hiuhu",
    "Hodari",
    "Hollis",
    "Homer",
    "Horace",
    "Horacio",
    "Houston",
    "Howard",
    "Hubert",
    "Hugh",
    "Hugo",
    "Humberto",
    "Humud",
    "Hung",
    "Hunter",
    "Hurani",
    "Husani",
    "Huseni",
    "Husni",
    "Ian",
    "Ibada",
    "Idi",
    "Idriis",
    "Ignacio",
    "Ikinya",
    "Imaan",
    "Imamu",
    "Iman",
    "Imara",
    "Imarika",
    "Ira",
    "Iregi",
    "Irungu",
    "Irvin",
    "Irving",
    "Irwin",
    "Isa",
    "Isaac",
    "Isaam",
    "Isaiah",
    "Isiah",
    "Isidro",
    "Isilahi",
    "Isimo",
    "Isiyoshindika",
    "Islam",
    "Ismael",
    "Ismaili",
    "Israel",
    "Issa",
    "Issac",
    "Ita",
    "Itimu",
    "Ivan",
    "Ivory",
    "Jaafar",
    "Jabali",
    "Jabari",
    "Jabir",
    "Jabiri",
    "Jack",
    "Jackson",
    "Jacob",
    "Jacques",
    "Jafari",
    "Jaha",
    "Jahi",
    "Jahina",
    "Jake",
    "Jalali",
    "Jalil",
    "Jalili",
    "Jamaal",
    "Jamal",
    "Jamali",
    "Jamar",
    "Jame",
    "Jamel",
    "James",
    "Jamil",
    "Jaramogi",
    "Jared",
    "Jarred",
    "Jarrett",
    "Jarrod",
    "Jarvis",
    "jasiri",
    "Jason",
    "Jasper",
    "Jauhar",
    "Javier",
    "Jawaad",
    "Jay",
    "Jayson",
    "Jecha",
    "Jed",
    "Jefar",
    "Jeff",
    "Jefferey",
    "Jefferson",
    "Jeffery",
    "Jeffrey",
    "Jeffry",
    "Jela",
    "Jelani",
    "Jemadari",
    "Jenebi",
    "Jengo",
    "Jerald",
    "Jeremiah",
    "Jeremy",
    "Jermaine",
    "Jerold",
    "Jerome",
    "Jerrod",
    "Jerrold",
    "Jess",
    "Jesse",
    "Jesus",
    "Jim",
    "Jimiyu",
    "Jimmy",
    "jitu",
    "Jitujeusi",
    "Joaquin",
    "Joe",
    "Joel",
    "Joesph",
    "Joey",
    "John",
    "Johnathan",
    "Johnathon",
    "Johnie",
    "Johnny",
    "Jomo",
    "Jon",
    "Jonah",
    "Jonas",
    "Jonathan",
    "Jonathon",
    "Jorge",
    "Jose",
    "Josef",
    "Joseph",
    "Josh",
    "Joshua",
    "Josiah",
    "Josue",
    "Juan",
    "Judson",
    "Jules",
    "Julian",
    "Julio",
    "Julius",
    "Juma",
    "Jumaane",
    "Jumanne",
    "Jumba",
    "jumbe",
    "Junior",
    "Justin",
    "Juta",
    "Kabaila",
    "Kabona",
    "Kadhi",
    "Kadiri",
    "Kafara",
    "Kafil",
    "Kagai",
    "Kagunda",
    "Kahiga",
    "Kahini",
    "Kairu",
    "Kalamka",
    "Kamau",
    "Kame",
    "Kamil",
    "Kamili",
    "Kanaifu",
    "Kandoro",
    "Kani",
    "Kanja",
    "Kanoro",
    "Karama",
    "Karani",
    "Kareem",
    "Karega",
    "Karim",
    "Kariuki",
    "Karl",
    "Karume",
    "Kasim",
    "Kasisi",
    "Keith",
    "Kelvin",
    "Ken",
    "Kendall",
    "Kendrick",
    "Kenneth",
    "Kennith",
    "Kenny",
    "Kent",
    "Kenton",
    "Kermit",
    "Kesi",
    "Keto",
    "Keven",
    "Kevin",
    "Khalafu",
    "Khalfan",
    "Khalfani",
    "Khalifa",
    "Khalil",
    "Khamisi",
    "Khatibu",
    "Khelefu",
    "Kheri",
    "Khiari",
    "Kiambiroiro",
    "Kiango",
    "Kianjahe",
    "Kiano",
    "Kianyandaarwa",
    "Kiara",
    "Kibasila",
    "Kibasira",
    "Kibwana",
    "Kibwe",
    "Kieth",
    "Kifimbo",
    "Kigoma",
    "Kihara",
    "Kihiga",
    "Kijana",
    "Kijani",
    "Kimameta",
    "Kimathi",
    "Kimotho",
    "Kimweri",
    "Kimya",
    "Kinda",
    "Kinjeketile",
    "Kinjikitile",
    "Kinoro",
    "Kinyua",
    "Kioi",
    "Kiongozi",
    "Kip",
    "Kipanga",
    "Kirby",
    "Kirimi",
    "Kirinyaga",
    "Kirk",
    "Kisasi",
    "Kithingi",
    "Kitunda",
    "Kitunzi",
    "Kitwana",
    "Kiume",
    "Koigi",
    "Kombo",
    "Komboa",
    "Kondo",
    "Kongoresi",
    "Kory",
    "Kristo",
    "Kristopher",
    "Kudumu",
    "Kuguru",
    "Kumbufu",
    "Kumbuka",
    "Kunjufu",
    "Kurt",
    "Kurtis",
    "Kutisha",
    "Kuumba",
    "Kuweza",
    "Kwagalana",
    "Kwanza",
    "Kweli",
    "Kyle",
    "Lali",
    "Lamar",
    "Lamont",
    "Lance",
    "Landon",
    "Lane",
    "Lanny",
    "Larry",
    "Latif",
    "Laurence",
    "Lavern",
    "Lawrence",
    "Lazaro",
    "Leland",
    "Lemuel",
    "Lenard",
    "Lenny",
    "Leo",
    "Leon",
    "Leonard",
    "Leonardo",
    "Leonel",
    "Leopoldo",
    "Leroy",
    "Les",
    "Lester",
    "Levi",
    "Lewis",
    "Lincoln",
    "Linwood",
    "Lionel",
    "Lipapwiche",
    "Liyongo",
    "Lloyd",
    "Logan",
    "Lon",
    "Lonnie",
    "Loren",
    "Lorenzo",
    "Louie",
    "Louis",
    "Lowell",
    "Loyd",
    "Lucas",
    "Luciano",
    "Lucien",
    "Lucio",
    "Luis",
    "Luke",
    "Lumumba",
    "Luther",
    "Lyle",
    "Lyman",
    "Maabade",
    "Maalik",
    "Maamuni",
    "Maarifa",
    "Mabruke",
    "Mac",
    "Macaria",
    "Machano",
    "Machungwa",
    "Machupa",
    "Mack",
    "Madhubuti",
    "Magoma",
    "Mahbub",
    "Mahdi",
    "Mahfudh",
    "Mahiri",
    "Mahmud",
    "Maina",
    "Maitho",
    "Majaliwa",
    "Majid",
    "Major",
    "Majuto",
    "Makalani",
    "Makame",
    "Makamu",
    "Makini",
    "Makori",
    "Makungu",
    "Makwetu",
    "Malcolm",
    "Maliik",
    "Malik",
    "Maliki",
    "Mambo",
    "Manani",
    "Mandara",
    "Maneno",
    "Manuel",
    "Manzi",
    "Manzili",
    "Mapute",
    "Marc",
    "Marcel",
    "Marcelino",
    "Marcelo",
    "Marco",
    "Marcos",
    "Marcus",
    "Margarito",
    "Mariano",
    "Mario",
    "Mark",
    "Marlin",
    "Marlon",
    "Marquis",
    "Marshall",
    "Martin",
    "Marty",
    "Marvin",
    "Marzuku",
    "Masamaha",
    "Mashal",
    "Mashuhuri",
    "Masilahi",
    "Maskini",
    "Mason",
    "Masud",
    "Matari",
    "Mathaathi",
    "Mathew",
    "Matojo",
    "Matt",
    "Matthew",
    "Matu",
    "Matwa",
    "Maulana",
    "Maulidi",
    "Maundu",
    "Maurice",
    "Mauricio",
    "Mauro",
    "Mawagali",
    "Max",
    "Maxwell",
    "Maynard",
    "Mbaarak",
    "Mbamba",
    "Mbaruku",
    "Mbaya",
    "Mbingu",
    "Mbishiri",
    "Mbita",
    "Mbogo",
    "Mbui",
    "Mbwana",
    "Mchawi",
    "Mcheshi",
    "Mdahoma",
    "Mdogo",
    "Mel",
    "Melvin",
    "Merlin",
    "Merrill",
    "Mervin",
    "Mfaki",
    "Mfalme",
    "Mgeni",
    "Mhina",
    "Miano",
    "Micah",
    "Michael",
    "Micheal",
    "Michel",
    "Mickey",
    "Miguel",
    "Migwi",
    "Mike",
    "Miles",
    "Milford",
    "Milina",
    "Millard",
    "Milo",
    "Milton",
    "Mirza",
    "Mitch",
    "Mitchel",
    "Mitchell",
    "Miujiza",
    "Mjibu",
    "Mjima",
    "Mkadam",
    "Mkamba",
    "Mkristo",
    "Mkubwa",
    "Mkufunzi",
    "Mkwasi",
    "Mohamed",
    "Mohammad",
    "Mohammed",
    "Moises",
    "Mokabi",
    "Monroe",
    "Monte",
    "Monty",
    "Morani",
    "Morris",
    "Morton",
    "Moses",
    "Mosi",
    "Mpenda",
    "Mrehe",
    "Mrekhe",
    "Msamaki",
    "Msanaa",
    "Msemaji",
    "Mshabaha",
    "Mshangama",
    "Mshindi",
    "Mtaalamu",
    "Mtafiti",
    "Mtangulizi",
    "Mtavila",
    "Mtawa",
    "Mtembei",
    "Mteremeshi",
    "Mteremo",
    "Mteule",
    "Mtoro",
    "Mtoto",
    "Mtulivu",
    "Mtume",
    "Mtumwa",
    "Mubaadar",
    "Mudrik",
    "Mufid",
    "Muga",
    "Mugendi",
    "Mugi",
    "Mugo",
    "Muhamadi",
    "Muhammad",
    "Muhammed",
    "Muhashaham",
    "Muhashmy",
    "Muhidini",
    "Muhsin",
    "Muhyiddin",
    "Muiru",
    "Muiruri",
    "Mukanda",
    "Mukhtaar",
    "Mukhwana",
    "Mukiri",
    "Mukuru",
    "Mundhir",
    "Munene",
    "Mungu",
    "Munim",
    "Munir",
    "Munoru",
    "Muombwa",
    "Muraguri",
    "Murathi",
    "Muraya",
    "Muriithi",
    "Murimi",
    "Muriu",
    "Muriuki",
    "Muroki",
    "Murray",
    "Murtada",
    "Muruthi",
    "Musa",
    "Mustafa",
    "Mutahi",
    "Mutegi",
    "Mutemi",
    "Muthee",
    "Muthemba",
    "Muthengi",
    "Mutheru",
    "Muthomi",
    "Muthuri",
    "Mutira",
    "Mutiri",
    "Mutitu",
    "Mutonyi",
    "Mutua",
    "Muturi",
    "Muugi",
    "Muumin",
    "Muyaka",
    "Mwaka",
    "Mwalimu",
    "Mwanga",
    "Mwangi",
    "Mwanza",
    "Mwapacha",
    "Mwara",
    "Mwarania",
    "Mwaria",
    "Mwenda",
    "Mwinyi",
    "Mwinyimadi",
    "Mwita",
    "Myles",
    "Myron",
    "Mzale",
    "Mzee",
    "Naadir",
    "Naasir",
    "Nabhani",
    "Nabil",
    "Nadhim",
    "Najub",
    "Nangila",
    "Nangwaya",
    "Nanji",
    "Napoleon",
    "Nasiir",
    "Nasila",
    "Nasor",
    "Nassor",
    "Nathan",
    "Nathaniel",
    "Nathari",
    "Ndegwa",
    "Ndemi",
    "Nderu",
    "Ndwiga",
    "Neal",
    "Ned",
    "Neil",
    "Nelson",
    "Nemsi",
    "Nestor",
    "Newton",
    "Ngare",
    "Ngigi",
    "Ngumo",
    "Nguvumali",
    "Niamoja",
    "Nicholas",
    "Nick",
    "Nickolas",
    "Nicolas",
    "Nigel",
    "Njama",
    "Njata",
    "Njau",
    "Njiraini",
    "Njiru",
    "Njogu",
    "Njora",
    "Njowga",
    "Njururi",
    "Noah",
    "Noe",
    "Noel",
    "Nolan",
    "Norbert",
    "Norberto",
    "Norman",
    "Normand",
    "Norris",
    "Numbers",
    "Nuru",
    "Nyachae",
    "Nyaga",
    "Nyamu",
    "Nyatui",
    "Nyoike",
    "Nyuni",
    "Obuya",
    "Ochieng",
    "Octavio",
    "Odell",
    "Odikinyi",
    "Odis",
    "Odongo",
    "Odour",
    "Ohon",
    "Ojwang",
    "Okello",
    "Okeyo",
    "Okoth",
    "Olen",
    "Olin",
    "Oliver",
    "Oluoch",
    "Oman",
    "Omar",
    "Omari",
    "Omariba",
    "Omondi",
    "Omwancha",
    "Ongweng",
    "Onindo",
    "Onkwani",
    "Onyango",
    "Opiyo",
    "Orlando",
    "Orval",
    "Orville",
    "Oscar",
    "Osogo",
    "Osvaldo",
    "Othiamba",
    "Othiambo",
    "Otieno",
    "Otis",
    "Otto",
    "Ouma",
    "Owen",
    "Owino",
    "Owiti",
    "Owuor",
    "Pablo",
    "Pandu",
    "Parker",
    "Pasquale",
    "Patrick",
    "Paul",
    "Pedro",
    "Percy",
    "Perry",
    "Pete",
    "Peter",
    "Phil",
    "Philip",
    "Phillip",
    "Pierre",
    "Pili",
    "Pongwa",
    "Popo",
    "Porfirio",
    "Preston",
    "Prince",
    "Quentin",
    "Quincy",
    "Quinn",
    "Quinton",
    "Rafael",
    "Rafiki",
    "Raha",
    "Rahim",
    "Rajabu",
    "Ralph",
    "Ramadhani",
    "Ramiro",
    "Ramon",
    "Randal",
    "Randall",
    "Randell",
    "Randolph",
    "Randy",
    "Raphael",
    "Rashaad",
    "Rashad",
    "Rashid",
    "Rashidi",
    "Rasul",
    "Rauf",
    "Raul",
    "Ray",
    "Raymond",
    "Raymundo",
    "Reed",
    "Refugio",
    "Reggie",
    "Reginald",
    "Rehani",
    "Reid",
    "Reinaldo",
    "Rejalla",
    "Reuben",
    "Rex",
    "Reyes",
    "Reynaldo",
    "Ricardo",
    "Rich",
    "Richard",
    "Richie",
    "Rick",
    "Rickey",
    "Rickie",
    "Ricky",
    "Rico",
    "Ridha",
    "Ridhwani",
    "Rifaa",
    "Rigoberto",
    "Riitho",
    "Rijaal",
    "Riley",
    "Rob",
    "Robby",
    "Robert",
    "Roberto",
    "Robt",
    "Rocco",
    "Rocky",
    "Rod",
    "Roderick",
    "Rodger",
    "Rodney",
    "Rodolfo",
    "Rodrick",
    "Rodrigo",
    "Rogelio",
    "Roger",
    "Roland",
    "Rolando",
    "Rolland",
    "Roman",
    "Romeo",
    "Ron",
    "Ronald",
    "Ronnie",
    "Ronny",
    "Roosevelt",
    "Rory",
    "Roscoe",
    "Rosendo",
    "Ross",
    "Roy",
    "Royal",
    "Royce",
    "Rubama",
    "Rubanza",
    "Ruben",
    "Rubin",
    "Rudolph",
    "Rudy",
    "Rufus",
    "Ruguru",
    "Ruhiu",
    "Rumaliza",
    "Runo",
    "Rupert",
    "Russ",
    "Russel",
    "Russell",
    "Rusty",
    "Ryan",
    "Saad",
    "Saalim",
    "Saami",
    "Saburi",
    "Sadiki",
    "Saeed",
    "Safari",
    "Safwani",
    "Saghiri",
    "Sahalani",
    "Said",
    "Sal",
    "Salaam",
    "Salah",
    "Saleh",
    "Salehe",
    "Salim",
    "Salmini",
    "Salvador",
    "Salvatore",
    "Sam",
    "Samir",
    "Sammie",
    "Sammy",
    "Samuel",
    "Sanford",
    "Sang",
    "Santiago",
    "Santos",
    "Sarahani",
    "Sarhaan",
    "Saul",
    "Scot",
    "Scott",
    "Scottie",
    "Scotty",
    "Sean",
    "Sebastian",
    "Sefu",
    "Seif",
    "Selemani",
    "Sergio",
    "Seth",
    "Seymour",
    "Shaabani",
    "Shaaboni",
    "Shahaab",
    "Shahidi",
    "Shakwe",
    "Shamakani",
    "Shambe",
    "Shane",
    "Sharif",
    "Shaun",
    "Shazidi",
    "Sheikh",
    "Sheldon",
    "Shelton",
    "Sherman",
    "Shibe",
    "Shibisha",
    "Shinda",
    "Shinuni",
    "Shomari",
    "Shujaa",
    "Shukrani",
    "Sidney",
    "Sifa",
    "Silas",
    "Simai",
    "Simba",
    "Simon",
    "Sinaan",
    "Siwatu",
    "Siwazuri",
    "Sokoro",
    "Solomon",
    "Son",
    "Songoro",
    "Sonny",
    "Spencer",
    "Stan",
    "Stanley",
    "Stefan",
    "Stephan",
    "Stephen",
    "Sterling",
    "Steve",
    "Steven",
    "Stevie",
    "Stewart",
    "Stima",
    "Stuart",
    "Sudi",
    "Suhuba",
    "Sultaan",
    "Sultan",
    "Sulubu",
    "Suluhu",
    "Sumai",
    "Sumait",
    "Sung",
    "Suwedi",
    "Suwesi",
    "Sydney",
    "Sylvester",
    "Taalib",
    "Tad",
    "Tafiti",
    "Taha",
    "Tajamali",
    "Taji",
    "Tajiri",
    "Takata",
    "Takatifu",
    "Taki",
    "Tanner",
    "Tarishi",
    "Tawfiki",
    "Tayari",
    "Ted",
    "Teddy",
    "Tendaji",
    "Teremesha",
    "Terence",
    "Terrance",
    "Terrell",
    "Terrence",
    "Thaabiti",
    "Thabiti",
    "Thad",
    "Thaddeus",
    "Thairu",
    "Thanh",
    "Thani",
    "Theodore",
    "Theron",
    "Thomas",
    "Thurman",
    "Thuweni",
    "Tiifu",
    "Tim",
    "Timmy",
    "Timothy",
    "Tindo",
    "Toby",
    "Tod",
    "Todd",
    "Tom",
    "Tomas",
    "Tommy",
    "Tony",
    "Tory",
    "Travis",
    "Trent",
    "Trenton",
    "Trevor",
    "Trey",
    "Trinidad",
    "Tristan",
    "Troy",
    "Truman",
    "Tukufu",
    "Tulivu",
    "Tumaini",
    "Tumbo",
    "Tuwile",
    "Ty",
    "Tyler",
    "Tyree",
    "Tyrone",
    "Tyson",
    "Ubaya",
    "Ubora",
    "Ubwa",
    "Ufanisi",
    "Uhuru",
    "Uki",
    "Ukurugenzi",
    "Uledi",
    "Ulysses",
    "Umar",
    "Urafiki",
    "Usaama",
    "Ushindi",
    "Valentin",
    "Van",
    "Vance",
    "Vaughn",
    "Vern",
    "Vernon",
    "Vicente",
    "Victor",
    "Vince",
    "Vincent",
    "Virgil",
    "Vito",
    "Vuai",
    "Wade",
    "Waga",
    "Waithaka",
    "Waitimu",
    "Waiyaki",
    "Wakili",
    "Waldo",
    "Walker",
    "Wallace",
    "Walter",
    "Wamai",
    "Wamugunda",
    "Wamukota",
    "Wamwara",
    "Wangombe",
    "Wangondu",
    "Wanyanga",
    "Ward",
    "Warren",
    "Waruhiu",
    "Warui",
    "Waweru",
    "Wayne",
    "Waziri",
    "Weldon",
    "Wendell",
    "Wesley",
    "Weston",
    "Wilbert",
    "Wilbur",
    "Wilburn",
    "Wiley",
    "Wilford",
    "Wilfred",
    "Wilfredo",
    "Will",
    "Willard",
    "William",
    "Williams",
    "Willis",
    "Wilmer",
    "Wilson",
    "Wilton",
    "Winfred",
    "Winston",
    "Woodrow",
    "Wyatt",
    "Xavier",
    "Yahya",
    "Yakubu",
    "Yasini",
    "Yong",
    "Yusuf",
    "Zachariah",
    "Zachary",
    "Zachery",
    "Zahir",
    "Zahor",
    "Zahran",
    "Zahur",
    "Zaki",
    "Zakwani",
    "Zalika",
    "Zamoyoni",
    "Zane",
    "Zende",
    "Zuber",
    "Zuberi",
    "Zuher",
    "Zuhri",
    "Zuri",

]


def get_random():
    return random.choice(NAMES)

